# Needs Nim Compiler Version 0.19.0 [Windows: i386]
TARGET=nucleo_f030r8

# GCC names
CC     = arm-none-eabi-gcc
SIZE   = arm-none-eabi-size
OBJDUMP= arm-none-eabi-objdump
OBJCOPY= arm-none-eabi-objcopy

# FOR GCC FLAGS
CFLAGS  = -mcpu=cortex-m0 -mthumb -ffunction-sections -fdata-sections -Os -g
CFLAGS  += -nostartfiles
CFLAGS  += -Wl,-T,$(LD_SCRIPT),-Map=$(TARGET).map,--gc-sections
CFLAGS  += --specs=nano.specs
CFLAGS  += -Isrc/sys
CFLAGS  += -Isrc/lib/xprintf

# Linker script
LD_SCRIPT = $(F0D)/STM32F030R8Tx_FLASH.ld

# For NIM
NMCD      = src/nimcache
NIM       = nim
NIMFLAGS += -d:release
NIMFLAGS += --opt:size
NIMFLAGS += --nimcache:$(NMCD)
NIMFLAGS += --path:src/
NIMFLAGS += --path:src/sys/
NIMFLAGS += --path:src/f0/
NIMFLAGS += --gc:none

NIMFLAGS += --os:standalone
NIMFLAGS += --cpu:arm
NIMFLAGS += --deadCodeElim:on
NIMFLAGS += --stacktrace:off
NIMFLAGS += --parallelBuild:0
NIMFLAGS += --verbosity:1


# Search path for source file
VPATH = src:\
	    src/sys:\
	    src/f0:\
	    src/lib/xprintf

# Default target
all: $(TARGET).elf
	@$(OBJDUMP) -hSC $< >       $(TARGET).lst
	@$(OBJCOPY) -O ihex $(<)    $(TARGET).hex
	@$(OBJCOPY) -O binary -S $< $(TARGET).bin
	@$(SIZE) $<
	"d:\STM32-ST-LINK-Utility\ST-LINK Utility\ST-LINK_CLI.exe" -c SWD -P $(TARGET).hex -Rst

SRC_NIM     = src/uart.nim \
	          src/pwm.nim \
			  src/gpio.nim \
			  src/spi.nim \
			  src/conf_sys.nim

MAIN_NIM    = src/main.nim

SYSD        = src/sys
F0D         = src/f0
SRC_SUB_NIM = $(wildcard $(SYSD)/*.nim)  $(wildcard $(F0D)/*.nim)
SRC_SUB_C   = $(wildcard $(SYSD)/*.c)    $(wildcard $(F0D)/*.c) src/lib/xprintf/xprintf.c

OBJECTS = $(addprefix $(NMCD)/,$(notdir $(SRC_SUB_C:%.c=%.o)))

# Explicitly reorder "syscalls.o" to avoid link error use with printf() function.
OBJECTS := $(filter-out $(NMCD)/syscalls.o,$(OBJECTS))
OBJECTS := $(NMCD)/syscalls.o $(OBJECTS)

$(TARGET).elf: $(MAIN_NIM) $(SRC_NIM) $(SRC_SUB_NIM) $(OBJECTS)
	$(NIM) c -o:$@ --passC:"$(CFLAGS)" --passL:"$(CFLAGS)" $(NIMFLAGS) --passL:"$(OBJECTS)" $<

$(NMCD)/%.o: %.c
	$(CC) $(CFLAGS) -c -MMD -MP -o $@ $^

$(NMCD)/%.o: %.s
	$(CC) $(CFLAGS) -c -o $@ $^

.PHONY: flash clean d

flash:
	echo $(TARGET).elf | xargs -I {} openocd -f interface/cmsis-dap.cfg -f target/stm32f3x.cfg -c 'init;program {};reset;exit'

clean:
	-@$(RM) $(filter-out $(NMCD)/dummy.txt,$(wildcard $(NMCD)/*))
	-@$(RM) $(TARGET).*

d:
	@echo OBJECTS
	@echo $(OBJECTS)
	@echo DEPS
	@echo $(DEPS)

###############################################################################
# Dependencies
###############################################################################
DEPS = $(OBJECTS:%.o=.%.d)
-include $(DEPS)

