import stm32f030x8
import gpio, spi
import conf_sys

#[
// Use PWM3-1,PWM3-2 with TIM3
#define PWM_PORT_LEFT_HI    PB_5    // TIM3_CH2
#define PWM_PORT_RIGHT_HI   PB_4    // TIM3_CH1
// Use PWM3-3,PWM3-4 with TIM3
#define PWM_PORT_LEFT_LOW   PC_9    // TIM3_CH4
#define PWM_PORT_RIGHT_LOW  PC_8    // TIM3_CH3
]#

const HERZ_44100 = 44100

proc pwm_period_timer_start*(){.inline.}=
    TIM3_IRQn.NVIC_EnableIRQ
#TIM3.EGR.bset(TIM_EGR_UG_Pos)     #

proc pwm_period_timer_stop*(){.inline.}=
    TIM3_IRQn.NVIC_disableIRQ
#TIM3.EGR.bclr(TIM_EGR_UG_Pos)     #

proc timer3seup()=
    RCC.APB1ENR.bset(RCC_APB1ENR_TIM3EN_Pos)           # TIM3 base clock enable
    TIM3.CR1.bset(TIM_CR1_CEN_Pos, TIM_CR1_ARPE_Pos)   # (CEN,ARPE) start countup
    TIM3.DIER.bset(TIM_DIER_UIE_Pos)                   # (UIE) Update 周期割り込み発生許可
    TIM3.PSC.st  0                                     # 分周しない
    #TIM3_IRQn.NVIC_EnableIRQ
    TIM3_IRQn.NVIC_SetPriority 0                       # 最高の割り込み優先度

proc pwm_dutyR_hi*(duty:auto){.inline.} =
    TIM3.CCR1.st duty
proc pwm_dutyL_hi*(duty:auto){.inline.} =
    TIM3.CCR2.st duty
proc pwm_dutyR_low*(duty:auto){.inline.} =
    TIM3.CCR3.st duty
proc pwm_dutyL_low*(duty:auto){.inline.} =
    TIM3.CCR4.st duty

proc pwmL_duty*(duty:auto){.inline.} =
    pwmDutyLHi(duty)
proc pwmR_duty*(duty:auto){.inline.} =
    pwmDutyRHi(duty)

proc setPwmPeriod*(freq:int)=
    TIM3.CCER.st 0       # ch1～ch4 出力禁止。 設定中は禁止にしておく。
    TIM3.ARR.st (SYSTEM_CLOCK div freq) # PWM周期設定
    TIM3.CCER.st 0x1111  # ch1～ch4 出力許可

proc initPwm*() {.used.} =
    timer3seup()
    block: # setup output port and pwm
        RCC.AHBENR.bset(RCC_AHBENR_GPIOBEN_Pos)    # GPIOB clock enable
        RCC.AHBENR.bset(RCC_AHBENR_GPIOCEN_Pos)    # GPIOC clock enable
        # PB4,AF1,TIM3_CH1/PWM
        # PB5,AF1,TIM3_CH2/PWM
        GPIOB.MODER.bset(GPIO_MODER_MODER4_Pos + 1,GPIO_MODER_MODER5_Pos + 1)
        # Select AF1 function -> PWM CHx
        GPIOB.AFR[0].bset(GPIO_AFRL_AFSEL4_Pos,GPIO_AFRL_AFSEL5_Pos)
        # PC8,AF0,TIM3_CH3/PWM
        # PC9,AF0,TIM3_CH4/PWM
        GPIOC.MODER.bset(GPIO_MODER_MODER8_Pos + 1,GPIO_MODER_MODER9_Pos + 1)
        # Select AF0 function -> PWM CHx
        # Default is AF0.
    #
    TIM3.CCER.st 0       # ch1～ch4 出力禁止。 設定中は禁止にしておく。
    # 設定 開始
    TIM3.ARR.st (SYSTEM_CLOCK div HERZ_44100) # PWM周期設定
    TIM3.CCMR1.st 0x6868 # CHxポートをPWM動作に設定
    TIM3.CCMR2.st 0x6868 # 4ch全部 PWM1設定
    # 設定 終了
    TIM3.CCER.st 0x1111  # ch1～ch4 出力許可
    # PWM タイマースタート
    TIM3.EGR.bset(TIM_EGR_UG_Pos)     # (UG) set counter = 0 when overflow

############################################
const TEST_PORT_ENABLE = true

proc TIM3_IRQHandler {.exportc.} =

    if TIM3.SR.bitIsSet(TIM_SR_UIF_Pos):
        when TEST_PORT_ENABLE:
            test_port(1)




        when TEST_PORT_ENABLE:
            test_port(0)

        # end interruput
        TIM3.SR.bclr(TIM_SR_UIF_Pos)
