import stm32f030x8
import systick
import gpio

#[  SPI1
    SPI_MOSI    = PA_7,
    SPI_MISO    = PA_6,
    SPI_SCK     = PA_5,
    SPI_CS      = PB_6, ]#
# forward definition
proc send*(spi:auto,data:uint8): uint8  {.used,inline.}
proc send*(spi:auto,data:uint16):uint16 {.used,inline.}

const CS_Pin = 6 # PB6

proc cs_on*() {.inline.} =
    GPIOB.atomic_set(CS_Pin)

proc cs_off*(){.inline.} =
    GPIOB.atomic_clr(CS_Pin)

proc spiEnable*() =
    SPI1.CR1.bset(SPI_CR1_SPE_Pos)

proc sdSpiEnable*(){.inline.} =
    spiEnable()


####################
# SPI device setting
####################
# 8bit send / receive
proc send*(spi:auto,data:uint8): uint8 {.used,inline.} =
    while spi.SR.bitIsClr(SPI_SR_TXE_Pos):
        discard
    spi.DR.st8 data   # 8bit transfer
    while spi.SR.bitIsClr(SPI_SR_RXNE_Pos):
        discard
    return spi.DR.ld8   # 8bit receive

# 16bit send / receive (no check, so far)
proc send*(spi:auto,data:uint16): uint16 {.used,inline.} =
    # It must be set and confirm,
    # CR2_DS = 0x07 # 16bit data width
    # CR2_FRXT = 0  # RXNE event
    #
    # set 16bit data size
    spi.CR2.bset( SPI_CR2_DS_Pos+3)
    spi.CR2.bclr( SPI_CR2_FRXTH_Pos)

    while spi.SR.bitIsClr(SPI_SR_TXE_Pos):
        discard
    spi.DR.st16 data  # 16bit transfer
    while spi.SR.bitIsClr(SPI_SR_RXNE_Pos):
        discard
    result = spi.DR.ld16         # 16bit receive
    # set 8bit data size
    spi.CR2.bclr( SPI_CR2_DS_Pos+3)
    spi.CR2.bset( SPI_CR2_FRXTH_Pos)

proc initSPI*(spi:auto){.used.} =
    # PB6(CS) chip select port
    RCC.AHBENR.bset(RCC_AHBENR_GPIOBEN_Pos)  # GPIOB clock enable
    GPIOB.MODER.bset(GPIO_MODER_MODER6_Pos)

    if spi==SPI1:
        # spi port setting
        RCC.APB2ENR.bset(RCC_APB2ENR_SPI1EN_Pos)          # SPI1  clock enable
        RCC.AHBENR.bset(RCC_AHBENR_GPIOAEN_Pos)           # GPIOA clock enable
        GPIOA.MODER.bset(   GPIO_MODER_MODER5_Pos+1,      # PA5(SCK)
                            GPIO_MODER_MODER6_Pos+1,      # PA6(MISO)
                            GPIO_MODER_MODER7_Pos+1)      # PA7(MOSI)
        GPIOA.OSPEEDR.bset( GPIO_OSPEEDR_OSPEEDR5_Pos,
                            GPIO_OSPEEDR_OSPEEDR5_Pos+1,
                            GPIO_OSPEEDR_OSPEEDR6_Pos,
                            GPIO_OSPEEDR_OSPEEDR6_Pos+1,
                            GPIO_OSPEEDR_OSPEEDR7_Pos,
                            GPIO_OSPEEDR_OSPEEDR7_Pos+1)
        GPIOA.PUPDR.bset(   GPIO_PUPDR_PUPDR5_Pos+1,      # PA5(SCK)  Pull.down
                            GPIO_PUPDR_PUPDR6_Pos)        # PA6(MISO) Pull.up
                                                          # PA7(MOSI) P.P default
    when false:
        if spi==SPI2:
            # spi port setting
            RCC.APB1ENR.bset(RCC_APB1ENR_SPI2EN_Pos)      # SPI2  clock enable
            RCC.AHBENR.bset(RCC_AHBENR_GPIOBEN_Pos)       # GPIOB clock enable
            GPIOB.MODER.bset(   GPIO_MODER_MODER13_Pos+1, # PB13(SCK)
                                GPIO_MODER_MODER14_Pos+1, # PB14(MISO)
                                GPIO_MODER_MODER15_Pos+1) # PB15(MOSI)
            GPIOB.OSPEEDR.bset( GPIO_OSPEEDR_OSPEEDR13_Pos,
                                GPIO_OSPEEDR_OSPEEDR13_Pos+1,
                                GPIO_OSPEEDR_OSPEEDR14_Pos,
                                GPIO_OSPEEDR_OSPEEDR14_Pos+1,
                                GPIO_OSPEEDR_OSPEEDR15_Pos,
                                GPIO_OSPEEDR_OSPEEDR15_Pos+1)
            GPIOB.PUPDR.bset(   GPIO_PUPDR_PUPDR13_Pos+1,  # PA5(SCK)  Pull.down
                                GPIO_PUPDR_PUPDR14_Pos)    # PA6(MISO) Pull.up
                                                           # PA7(MOSI) P.P default
    #######################
    # SPI register settings
    #######################
    spi.CR1.bset(SPI_CR1_MSTR_Pos)
    spi.CR1.bset(SPI_CR1_BR_Pos) # sck=12MHz

    # NSS_SOFT [Important setting] or set SSOE=1
    spi.CR1.bset(SPI_CR1_SSM_Pos, SPI_CR1_SSI_Pos)

    # [Important setting if used 8bit transfer]
    spi.CR2.bset(SPI_CR2_FRXTH_Pos)

    # Enable SPI output
    # spi.CR1.bset(SPI_CR1_SPE_Pos)


