import stm32f030x8
import conf_sys


proc initSysclock48mhz*() =                 # For STM32F0 series, PLL settings
    if RCC.CFGR.bitIsSet(1):                # (1) Test if PLL is used as System clock
        RCC.CFGR.bclr(1,0)                  # (2) Select HSI as system clock
        while not RCC.CFGR.bitIsClr(3,2):
            discard                         # (3) Wait for switched to HSI
    RCC.CR.bclr(RCC_CR_PLLON_Pos)           # (4) Disable the PLL
    while RCC.CR.bitIsSet(RCC_CR_PLLRDY_Pos):
        discard                             # (5) Wait until PLLRDY is cleared
    # set parameters
    RCC.CFGR.bclr(21,20,19,18)
    RCC.CFGR.bset(21,19)                    # (6) Set the PLL multiplier to 12
    RCC.CR.bset(RCC_CR_PLLON_Pos)           # (7) Enable PLL
    while RCC.CR.bitIsClr(RCC_CR_PLLRDY_Pos):
        discard                             # (8) Wait until PLLRDY is set
    RCC.CFGR.bset(1)                        # (9) Select PLL as system clock
    while RCC.CFGR.bitIsClr(3):             # (10) Wait until the PLL is switched on
        discard

when false:
    proc initSystick*() = # Direct setting without library
        const SYSTICK_RELOAD_VAL   = ( SYSTEM_CLOCK  div 1000 ) - 1 # 1msec
        # use cpu clock = SYSTEM_CLOCK
        stm32f0_core.SysTick.LOAD.st SYSTICK_RELOAD_VAL
        stm32f0_core.SysTick.VAL.st 0    # clear counter
        # bit2: Use cpu clock
        # bit0: Count down timer start
        # bit1: Enable Systick interrupt
        stm32f0_core.Systick.CTRL.bset(2,0,1)

        SysTick_IRQn.NVIC_EnableIRQ
        SysTick_IRQn.NVIC_SetPriority 0

var tickcounter {.volatile.}= 0

proc getTickCounter*() :int=
    tickcounter

proc setTickCounter*(ms:int)=
    tickcounter = ms

proc wait_ms*(ms:int)=
   let prev = tickcounter
   while tickcounter < prev + ms:
        discard

proc SysTick_Handler {.exportc.} = # Called every 1msec
        inc(tickcounter)

